﻿using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Web.Utilities;
using Exam.Trains;
using Hotline.Exams.Trains;
using Hotline.Share.Requests.Train;
using JiebaNet.Segmenter.Common;
using System.Linq.Expressions;

namespace Hotline.Application.Exam.QueryExtensions.Trains
{
    public static class TrainPlanQueryExtensions
    {
        public static Expression<Func<TrainPlan,bool>> GetExpression(this TrainPlanPagedRequest trainPlanPagedRequest)
        {
            Expression<Func<TrainPlan, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<TrainPlan>()
                .AndIF(trainPlanPagedRequest.Code.IsNotNullOrEmpty(), x => x.Code.Contains(trainPlanPagedRequest.Code))
                .AndIF(trainPlanPagedRequest.Name.IsNotNullOrEmpty(), x => x.Name.Contains(trainPlanPagedRequest.Name))
                .AndIF(trainPlanPagedRequest.MaxStartTime.IsNotNull(), x => x.TrainStartTime <= trainPlanPagedRequest.MaxStartTime)
                .AndIF(trainPlanPagedRequest.MinStartTime.IsNotNull(), x => x.TrainEndTime >= trainPlanPagedRequest.MinStartTime)
                .ToExpression();

            return expression;
        }

        public static Expression<Func<TrainPlanTemplate,bool>> GetTemplateExpression(this TrainPlanPagedRequest trainPlanPagedRequest)
        {
            Expression<Func<TrainPlanTemplate, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<TrainPlanTemplate>()
                .AndIF(trainPlanPagedRequest.TrainTemplateIds.IsNotNull(), x => trainPlanPagedRequest.TrainTemplateIds.Contains(x.TraimTemplateId))
                .ToExpression();

            return expression;
        }
    }
}
