﻿using DocumentFormat.OpenXml.Office2010.Excel;
using Exam.Application.Interface.TestPapers;
using Exam.ExamManages;
using Exam.Infrastructure.Data.Entity;
using Exam.Infrastructure.Enums;
using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Web.Extensions;
using Exam.Infrastructure.Web.Utilities;
using Exam.Insfrastructure.Service.Service;
using Exam.Questions;
using Exam.Repository.Sqlsugar;
using Exam.Repository.Sqlsugar.Repositories;
using Exam.Share.ViewResponses.TestPaper;
using Exam.TestPapers;
using Hotline.Application.Exam.Core.Extensions;
using Hotline.Application.Exam.Extensions;
using Hotline.Application.Exam.QueryExtensions.TestPapers;
using Hotline.Exams.TestPapers;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Repository.SqlSugar.Exam.Interfaces.Questions;
using Hotline.Repository.SqlSugar.Exam.Interfaces.TestPapers;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Dtos.Questions;
using Hotline.Share.Dtos.TestPapers;
using Hotline.Share.Dtos.Trains;
using Hotline.Share.Requests.Exam;
using Hotline.Share.Tools;
using Hotline.Share.ViewResponses.Exam;
using JiebaNet.Segmenter.Common;
using MapsterMapper;
using NPOI.SS.Formula.Functions;
using SqlSugar;
using System.Collections.Immutable;
using XF.Domain.Dependency;

namespace Hotline.Application.Exam.Service.TestPapers
{
    public class TestPaperService : ApiService<TestPaper, AddTestPaperDto, UpdateTestPaperDto, HotlineDbContext>, ITestPaperService, IScopeDependency
    {
        private readonly ITestPaperRepository _repository;
        private readonly ITestPaperItemRepository _testPaperItemRepository;
        private readonly ITestPaperRuleRepository _testPaperRuleRepository;
        private readonly ITestPaperRuleTagRepository _testPaperRuleTagRepository;
        private readonly ITestPaperItemAnswerRepository _testPaperItemAnswerRepository;
        private readonly ITestPaperItemOptionsRepository _testPaperItemOptionsRepository;
        private readonly ITestPaperItemSourcewareRepository _testPaperItemSourcewareRepository;
        private readonly ITestPaperItemKnowladgeRepository _testPaperItemKnowladgeRepository;
        private readonly IDataPermissionFilterBuilder _dataPermissionFilterBuilder;
        private readonly IServiceProvider _serviceProvider;
        private readonly IMapper _mapper;

        public TestPaperService(ITestPaperRepository repository,
            ITestPaperItemRepository testPaperItemRepository,
            ITestPaperRuleRepository testPaperRuleRepository,
            ITestPaperRuleTagRepository testPaperRuleTagRepository,
            ITestPaperItemAnswerRepository testPaperItemAnswerRepository,
            ITestPaperItemOptionsRepository testPaperItemOptionsRepository,
            ITestPaperItemSourcewareRepository testPaperItemSourcewareRepository,
            ITestPaperItemKnowladgeRepository testPaperItemKnowladgeRepository,
            IDataPermissionFilterBuilder dataPermissionFilterBuilder, IServiceProvider serviceProvider,
            IMapper mapper) : base(repository, mapper)
        {
            _repository = repository;
            _testPaperItemRepository = testPaperItemRepository;
            _testPaperRuleRepository = testPaperRuleRepository;
            _testPaperRuleTagRepository = testPaperRuleTagRepository;
            this._testPaperItemAnswerRepository = testPaperItemAnswerRepository;
            this._testPaperItemOptionsRepository = testPaperItemOptionsRepository;
            this._testPaperItemSourcewareRepository = testPaperItemSourcewareRepository;
            this._testPaperItemKnowladgeRepository = testPaperItemKnowladgeRepository;
            _dataPermissionFilterBuilder = dataPermissionFilterBuilder;
            _serviceProvider = serviceProvider;
            _mapper = mapper;
        }

        #region public method
        public async Task<TestPaperDto> GetAsync(EntityQueryRequest entityQueryRequest)
        {
            var entity = await _repository.GetAsync(entityQueryRequest.Id);

            var testPaperDto = _mapper.Map<TestPaperDto>(entity);

            testPaperDto.TestPaperItemDtos = await GetTestPaperItemDtos(entityQueryRequest);

            testPaperDto.TestPaperRuleDtos = await GetTestPaperRuleDtos(entityQueryRequest);

            return testPaperDto;
        }

        public async Task<(int, List<TestPaperViewResponse>)> GetListAsync(TestPaperPagedRequest queryRequest)
        {
            ISugarQueryable<TestPaperViewResponse> queryable = QueryResult(queryRequest);

            var result = await queryable.ToListAsync();

            var total = await queryable.CountAsync();

            return (total, result);
        }

        public async Task<PageViewResponse<TestPaperViewResponse>> GetPagedListAsync(TestPaperPagedRequest queryRequest)
        {
            ISugarQueryable<TestPaperViewResponse> queryable = QueryResult(queryRequest);

            var list = await queryable.ToPageListAsync(queryRequest.PageIndex, queryRequest.PageSize);
            var total = await queryable.CountAsync();

            var result = new TestPaperPageViewResponse
            {
                Items = list,
                Pagination = new Pagination(queryRequest.PageIndex, queryRequest.PageSize, total)
            };

            return result;
        }

        /// <summary>
        /// 新增试卷
        /// </summary>
        /// <param name="actionRequest"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public override async Task<string> AddAsync(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            var id = await base.AddAsync(actionRequest, cancellationToken);

            ResolveTestPaperId(actionRequest, id);

            await AddTestPaperItems(actionRequest, cancellationToken);

            await AddTestPaperRules(actionRequest, cancellationToken);

            await AddTestPaperRuleTags(actionRequest, cancellationToken);

            await AddTestPaperItemAnswer(actionRequest, cancellationToken);

            await AddTestPaperItemOptions(actionRequest, cancellationToken);

            await AddTestPaperItemKnowladgeAsync(actionRequest, cancellationToken);

            await AddTestPaperItemSourcewareAsync(actionRequest, cancellationToken);

            return id;
        }

        /// <summary>
        /// 修改试卷
        /// </summary>
        /// <param name="actionRequest"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public override async Task UpdateAsync(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            await base.UpdateAsync(actionRequest, cancellationToken);

            ResolveTestPaperId(actionRequest, actionRequest.Id);

            await ModifyTestPaperItems(actionRequest, cancellationToken);

            await ModifyTestPaperRules(actionRequest, cancellationToken);

            await ModifyTestPaperRuleTags(actionRequest, cancellationToken);

            await ModifyTestPaperItemAnswer(actionRequest, cancellationToken);

            await ModifyTestPaperItemOptions(actionRequest, cancellationToken);

            await ModifyTestPaperItemKnowladgeAsync(actionRequest, cancellationToken);

            await ModifyTestPaperItemSourcewareAsync(actionRequest, cancellationToken);
        }


        /// <summary>
        /// 删除试卷
        /// </summary>
        /// <param name="entityQueryRequest"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public override async Task DeleteAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            var testPaperRuleTags = await GetTestPaperRuleTags(entityQueryRequest);

            await base.DeleteAsync(entityQueryRequest, cancellationToken);

            var tempEntityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperItem>()
               .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.TestPaperId == entityQueryRequest.Id)
               .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.TestPaperId)).ToEntityQueryRequest<TestPaperItem>();

            await DeleteTestPaperItems(tempEntityQueryRequest, cancellationToken);

            tempEntityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperRule>()
                .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.TestPaperId == entityQueryRequest.Id)
                .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.TestPaperId)).ToEntityQueryRequest<TestPaperRule>();


            await DeleteTestPaperRules(tempEntityQueryRequest, cancellationToken);


            var ids = testPaperRuleTags.Select(x => x.Id).ToList();
            tempEntityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperRuleTag>()
                .AndIF(ids.IsNotNull(), x => ids.Contains(x.Id)).ToEntityQueryRequest<TestPaperRuleTag>();

            await DeleteTestPaperRuleTags(tempEntityQueryRequest, cancellationToken);
        }

        /// <summary>
        /// 获取试卷试题数量
        /// </summary>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        public async Task<List<TestPaperQuestionCountViewResponse>> GetTestPaperQuestionCount(TestPaperQuestionCountRequest testPaperQuestionCountRequest)
        {
            var expression = testPaperQuestionCountRequest.GetTestPaperCountExpression();
            var testPaper = await _repository.Queryable().Where(expression).FirstAsync();

            var testPaperQuestionCountViewResponses = new List<TestPaperQuestionCountViewResponse>();
            if (testPaper != null)
            {
                if(testPaper.Mode == Share.Enums.Exams.EExamMode.Random)
                {
                    testPaperQuestionCountViewResponses = await CalcuteRandomQuestionCount(testPaperQuestionCountRequest);
                }
                else if(testPaper.Mode == Share.Enums.Exams.EExamMode.Manual)
                {
                    testPaperQuestionCountViewResponses = await CalcuteManualQuestionCount(testPaperQuestionCountRequest);
                }
            }
            return testPaperQuestionCountViewResponses;
        }


        public async Task GenerateTestPaper(GenerateTestPaperRequest generateTestPaperRequest, CancellationToken cancellationToken)
        {
            var testPaper = await _repository.GetAsync(x => x.Id == generateTestPaperRequest.TestPaperId);

            if (testPaper != null)
            {
                if (testPaper.Mode == Share.Enums.Exams.EExamMode.Random)
                {
                    List<TestPaperItem> testPaperItems = await SyncQuestion(generateTestPaperRequest, cancellationToken);

                    await SyncQuestionOptions(testPaperItems, cancellationToken);
                    await SyncQuestionSourcewares(testPaperItems, cancellationToken);
                    await SyncQuestionKnowladge(testPaperItems, cancellationToken);
                    await SyncQuestionAnswer(testPaperItems, cancellationToken);

                }
            }
            
        }

        private async Task SyncQuestionAnswer(List<TestPaperItem> testPaperItems, CancellationToken cancellationToken)
        {
            var questionAnswersRepository = new ExamRepository<QuestionAnswer>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);
            var questionIds = testPaperItems.Select(x => x.QuestionId);
            var questionAnswers = await questionAnswersRepository.Queryable().Where(x => questionIds.Contains(x.QuestionId)).ToListAsync();

            var testPaperItemAnswers = new List<TestPaperItemAnswer>();

            questionAnswers.ForEach(item =>
            {
                var testPaperItemAnswer = _mapper.Map<TestPaperItemAnswer>(item);
                var testPaperItem = testPaperItems.FirstOrDefault(x => x.QuestionId == item.QuestionId);
                testPaperItemAnswer.QueswerAnswerId = item.Id;
                testPaperItemAnswer.TestPaperItemId = testPaperItem?.Id ?? string.Empty;
                testPaperItemAnswers.Add(testPaperItemAnswer);
            });

            testPaperItemAnswers.ToInsert();

            await _testPaperItemAnswerRepository.AddWithValidateAsync(testPaperItemAnswers, cancellationToken);
        }

        private async Task SyncQuestionKnowladge(List<TestPaperItem> testPaperItems, CancellationToken cancellationToken)
        {
            var questionKnowladgesRepository = new ExamRepository<QuestionKnowladge>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);
            var questionIds = testPaperItems.Select(x => x.QuestionId);
            var questionKnowladges = await questionKnowladgesRepository.Queryable().Where(x => questionIds.Contains(x.QuestionId)).ToListAsync();

            var testPaperItemKnowladges = new List<TestPaperItemKnowladge>();

            questionKnowladges.ForEach(item =>
            {
                var testPaperItemKnowladge = _mapper.Map<TestPaperItemKnowladge>(item);
                var testPaperItem = testPaperItems.FirstOrDefault(x => x.QuestionId == item.QuestionId);
                testPaperItemKnowladge.KnowladgeId = item.Id;
                testPaperItemKnowladge.TestPaperItemId = testPaperItem?.Id ?? string.Empty;
                testPaperItemKnowladges.Add(testPaperItemKnowladge);
            });

            testPaperItemKnowladges.ToInsert();

            await _testPaperItemKnowladgeRepository.AddWithValidateAsync(testPaperItemKnowladges, cancellationToken);
        }

        private async Task SyncQuestionSourcewares(List<TestPaperItem> testPaperItems, CancellationToken cancellationToken)
        {
            var questionSourcewaresRepository = new ExamRepository<QuestionSourceware>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);
            var questionIds = testPaperItems.Select(x => x.QuestionId);
            var questionSourcewares = await questionSourcewaresRepository.Queryable().Where(x => questionIds.Contains(x.QuestionId)).ToListAsync();

            var testPaperItemSourcewares = new List<TestPaperItemSourceware>();

            questionSourcewares.ForEach(item =>
            {
                var testPaperItemSourceware = _mapper.Map<TestPaperItemSourceware>(item);
                var testPaperItem = testPaperItems.FirstOrDefault(x => x.QuestionId == item.QuestionId);
                testPaperItemSourceware.SourcewareId = item.Id;
                testPaperItemSourceware.TestPaperItemId = testPaperItem?.Id ?? string.Empty;
                testPaperItemSourcewares.Add(testPaperItemSourceware);
            });

            testPaperItemSourcewares.ToInsert();

            await _testPaperItemSourcewareRepository.AddWithValidateAsync(testPaperItemSourcewares, cancellationToken);
        }

        private async Task SyncQuestionOptions(List<TestPaperItem> testPaperItems, CancellationToken cancellationToken)
        {
            var questionOptionsRepository =new ExamRepository<QuestionOptions>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);
            var questionIds = testPaperItems.Select(x => x.QuestionId);
            var questionOptions = await questionOptionsRepository.Queryable().Where(x=>questionIds.Contains(x.QuestionId)).ToListAsync();

            var testPaperItemOptions = new List<TestPaperItemOptions>();

            questionOptions.ForEach(item =>
            {
                var testPaperItemOption = _mapper.Map<TestPaperItemOptions>(item);
                var testPaperItem = testPaperItems.FirstOrDefault(x => x.QuestionId == item.QuestionId);
                testPaperItemOption.QuestionOptionId = item.Id;
                testPaperItemOption.TestPaperItemId = testPaperItem?.Id??string.Empty;
                testPaperItemOptions.Add(testPaperItemOption);
            });

            testPaperItemOptions.ToInsert();

            await _testPaperItemOptionsRepository.AddWithValidateAsync(testPaperItemOptions, cancellationToken);
        }

        private async Task<List<TestPaperItem>> SyncQuestion(GenerateTestPaperRequest generateTestPaperRequest, CancellationToken cancellationToken)
        {
            var questionRepository = new ExamRepository<Question>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);
            var questionTagRepository = new ExamRepository<QuestionTag>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);
            var testPaperRuleTable = _testPaperRuleRepository.Queryable().Where(x => x.TestPaperId == generateTestPaperRequest.TestPaperId);
            var testPaperTagTable = _testPaperRuleTagRepository.Queryable();
            var questionTagTable = questionTagRepository.Queryable();
            var questionTable = questionRepository.Queryable();

            var count = await testPaperRuleTable.Select(x => x.Count).FirstAsync();

            var questions = await questionTable.InnerJoin(questionTagTable, (q, qt) => q.Id == qt.QuestionId)
                .InnerJoin(testPaperTagTable, (q, qt, tt) => qt.TagId == tt.TagId)
                .InnerJoin(testPaperRuleTable, (q, qt, tt, tpr) => tt.TestPaperRuleId == tpr.Id)
                .Where((q, qt, tt, tpr)=>q.QuestionType == tpr.QuestionType)
                .Select((q, qt, tt, tpr) => q).Take(count).OrderBy(SqlFunc.GetRandom()).ToListAsync();

            var testPaperItems = new List<TestPaperItem>();

            questions.ForEach(item =>
            {
                var testPaperItem = _mapper.Map<TestPaperItem>(item);
                testPaperItem.TestPaperId = generateTestPaperRequest.TestPaperId;
                testPaperItem.QuestionId = item.Id;
                testPaperItems.Add(testPaperItem);
            });

            testPaperItems.ToInsert();

            await _testPaperItemRepository.AddWithValidateAsync(testPaperItems,cancellationToken);

            return testPaperItems;
        }
        #endregion

        #region private method
        /// <summary>
        /// 获取查询结果
        /// </summary>
        /// <param name="queryRequest"></param>
        /// <returns></returns>
        private ISugarQueryable<TestPaperViewResponse> QueryResult(TestPaperPagedRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var query = _repository.Queryable().Where(expression);
            var querable = query.OrderBy(o => o.SortIndex).Select(m => new TestPaperViewResponse
            {
                Name = m.Name,
                ExamType = m.ExamType,
                Mode = m.Mode,
                Code = m.Code,
                Id = m.Id,
                Status = m.Status,
                SortIndex = m.SortIndex,
                Remark = m.Remark,
            });
            return querable;
        }


        private async Task ModifyTestPaperItemOptions(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperItemDtos == null) return;

            var questionIds = actionRequest.TestPaperItemDtos.Where(x => x.QuestionType.CheckSelectType()).Select(x => x.QuestionId).ToList();

            var entityQueryRequest = new EntityQueryRequest
            {
                Expression = ExpressionableUtility.CreateExpression<TestPaperItemOptions>()
                             .AndIF(questionIds.IsNotNull(), x => questionIds.Contains(x.QuestionId)).ToExpression()
            };

            await DeleteTestPaperItemOptions(entityQueryRequest, cancellationToken);

            await AddTestPaperItemOptions(actionRequest, cancellationToken);

        }

        private async Task DeleteTestPaperItemOptions(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _testPaperItemOptionsRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task ModifyTestPaperItemAnswer(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperItemDtos == null) return;

            var questionIds = actionRequest.TestPaperItemDtos.Where(x => !x.QuestionType.CheckSelectType()).Select(x => x.QuestionId).ToList();

            var entityQueryRequest = new EntityQueryRequest
            {
                Expression = ExpressionableUtility.CreateExpression<TestPaperItemAnswer>()
                             .AndIF(questionIds.IsNotNull(), x => questionIds.Contains(x.QuestionId)).ToExpression()
            };

            await DeleteTestPaperItemAnswer(entityQueryRequest, cancellationToken);

            await AddTestPaperItemAnswer(actionRequest, cancellationToken);
        }

        private async Task DeleteTestPaperItemAnswer(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _testPaperItemAnswerRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task AddTestPaperRules(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            if (actionRequest.Mode == Share.Enums.Exams.EExamMode.Manual) return;

            actionRequest.TestPaperRuleDtos.ResolveOperationStatus();

            var testPaperRuleDtos = actionRequest.TestPaperRuleDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();
            var testPaperRules = _mapper.Map<List<TestPaperRule>>(testPaperRuleDtos);

            testPaperRules.ToInsert();


            await _testPaperRuleRepository.AddWithValidateAsync(testPaperRules, cancellationToken);

        }

        private async Task AddTestPaperItems(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperItemDtos == null) return;

            actionRequest.TestPaperItemDtos.ResolveOperationStatus();

            var testPaperItemDtos = actionRequest.TestPaperItemDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();
            var testPaperItems = _mapper.Map<List<TestPaperItem>>(testPaperItemDtos);
            //试题排序，为后续考试顺序出题做准备
            var index = 0;
            testPaperItems.ForEach(x =>
            {
                x.SortIndex = ++index;
            });

            testPaperItems.ToInsert();


            await _testPaperItemRepository.AddWithValidateAsync(testPaperItems, cancellationToken);
        }


        private async Task AddTestPaperRuleTags(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            actionRequest.TestPaperItemDtos.ResolveOperationStatus();

            var testPaperRuleDtos = actionRequest.TestPaperRuleDtos.Where(x => x.OperationStatus == EEOperationStatus.Add);

            var testPaperRuleTagDtos = testPaperRuleDtos.SelectMany(x => x.TestPaperRuleTagDtos).ToList();

            testPaperRuleTagDtos.ResolveOperationStatus();

            var testPaperRuleTags = _mapper.Map<List<TestPaperRuleTag>>(testPaperRuleTagDtos);

            testPaperRuleTags.ToInsert();

            await _testPaperRuleTagRepository.AddWithValidateAsync(testPaperRuleTags, cancellationToken);
        }

        private async Task ModifyTestPaperRules(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            if (actionRequest.Mode == Share.Enums.Exams.EExamMode.Manual) return;

            var all = await _testPaperRuleRepository.Queryable().Where(x => x.TestPaperId == actionRequest.Id).ToListAsync();

            actionRequest.TestPaperRuleDtos.ResolveOperationStatus(all);

            await AddTestPaperRules(actionRequest, cancellationToken);

            await UpdateTestPaperRules(actionRequest,all, cancellationToken);

            var entityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperRule>()
                .AndIF(actionRequest.Id.IsNotNullOrEmpty(), x => x.TestPaperId == actionRequest.Id).ToEntityQueryRequest<TestPaperRule>();

            await DeleteTestPaperRules(entityQueryRequest, cancellationToken);
        }

        private async Task DeleteTestPaperRules(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _testPaperRuleRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task UpdateTestPaperRules(UpdateTestPaperDto actionRequest,List<TestPaperRule> all, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            var testPaperRuleDtos = actionRequest.TestPaperRuleDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = testPaperRuleDtos.Select(x => x.Id);

            var testPaperRules = all.Where(x => ids.Contains(x.TestPaperId)).ToList();

            var entities = new List<TestPaperRule>();

            foreach (var testPaperRuleDto in testPaperRuleDtos)
            {
                var entity = testPaperRules.FirstOrDefault(x => x.Id == testPaperRuleDto.Id);

                entity = _mapper.Map(testPaperRuleDto, entity);

                entity.TestPaperId = actionRequest.Id;
            }

            entities.ToUpdate();

            await _testPaperRuleRepository.UpdateWithValidateAsync(entities, cancellationToken);

        }

        private async Task ModifyTestPaperRuleTags(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            var ruleIds = actionRequest.TestPaperRuleDtos.Select(x => x.Id);

            var all = await _testPaperRuleTagRepository.Queryable().Where(x => ruleIds.Contains(x.TestPaperRuleId)).ToListAsync();

            actionRequest.TestPaperRuleDtos.ResolveOperationStatus(all);

            await AddTestPaperRuleTags(actionRequest, cancellationToken);

            await UpdateTestPaperRuleTags(actionRequest, all, cancellationToken);

            var ids = actionRequest.TestPaperRuleDtos.Where(x => x.OperationStatus == EEOperationStatus.Delete).Select(x => x.Id).ToList();

            var entityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperRuleTag>()
                .AndIF(ids.IsNullOrEmpty(), x => ids.Contains(x.TestPaperRuleId)).ToEntityQueryRequest<TestPaperRuleTag>();

            await DeleteTestPaperRuleTags(entityQueryRequest, cancellationToken);
        }

        private async Task DeleteTestPaperRuleTags(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _testPaperRuleTagRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task UpdateTestPaperRuleTags(UpdateTestPaperDto actionRequest,List<TestPaperRuleTag> all, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            var ruleIds = actionRequest.TestPaperRuleDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).Select(x => x.Id).ToList();

            var testPaperRuleTags = all.Where(x => ruleIds.Contains(x.TestPaperRuleId)).ToList();

            var entities = new List<TestPaperRuleTag>();

            foreach (var testPaperRuleTagDto in actionRequest.TestPaperRuleDtos.SelectMany(x=>x.TestPaperRuleTagDtos))
            {
                var entity = testPaperRuleTags.FirstOrDefault(x => x.Id == testPaperRuleTagDto.Id);

                entity = _mapper.Map<UpdateTestPaperRuleTagDto, TestPaperRuleTag>(testPaperRuleTagDto,entity);
            }

            entities.ToUpdate();

            await _testPaperRuleTagRepository.UpdateWithValidateAsync(entities, cancellationToken);
        }

        private async Task ModifyTestPaperItems(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperItemDtos == null) return;

            var all = await _testPaperItemRepository.Queryable().Where(x => x.TestPaperId == actionRequest.Id).ToListAsync();

            actionRequest.TestPaperItemDtos.ResolveOperationStatus(all);

            await AddTestPaperItems(actionRequest, cancellationToken);

            await UpdateTestPaperItems(actionRequest,all, cancellationToken);

            var entityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperItem>()
                .AndIF(actionRequest.Id.IsNotNullOrEmpty(), x => x.TestPaperId == actionRequest.Id).ToEntityQueryRequest<TestPaperItem>();

            await DeleteTestPaperItems(entityQueryRequest, cancellationToken);
        }

        private async Task DeleteTestPaperItems(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _testPaperItemRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task UpdateTestPaperItems(UpdateTestPaperDto actionRequest,List<TestPaperItem> all, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperItemDtos == null) return;

            var testPaperItemDtos = actionRequest.TestPaperItemDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = testPaperItemDtos.Select(x => x.Id);

            var testPaperItems = all.Where(x => ids.Contains(x.TestPaperId)).ToList();

            var entities = new List<TestPaperItem>();

            foreach (var testPaperItemDto in testPaperItemDtos)
            {
                var entity = testPaperItems.FirstOrDefault(x => x.Id == testPaperItemDto.Id);

                entity = _mapper.Map(testPaperItemDto, entity);

                entity.TestPaperId = actionRequest.Id;
            }

            entities.ToUpdate();

            await _testPaperItemRepository.UpdateWithValidateAsync(entities, cancellationToken);
        }

        /// <summary>
        /// 获取试卷规则
        /// </summary>
        /// <param name="entityQueryRequest"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        private async Task<List<TestPaperRuleDto>> GetTestPaperRuleDtos(EntityQueryRequest entityQueryRequest)
        {
            var testPaperRuleTable = _testPaperRuleRepository.Queryable().Where(x => x.TestPaperId == entityQueryRequest.Id);

            var testPaperRuleTags = await GetTestPaperRuleTags(entityQueryRequest);

            var queryable = testPaperRuleTable.Select(t => new TestPaperRuleDto
            {
                Id = t.Id,
                TestPaperId = t.TestPaperId,
                Count = t.Count,
                DifficultyLevel = t.DifficultyLevel,
                SortIndex = t.SortIndex,
                QuestionType = t.QuestionType,
            });

            var testPaperRuleTagDtos = _mapper.Map<List<TestPaperRuleTagDto>>(testPaperRuleTags);
            var testPaperRuleList = await queryable.ToListAsync();
            testPaperRuleList.ForEach(x => x.TestPaperRuleTagDtos = testPaperRuleTagDtos.Where(n => n.TestPaperRuleId == x.Id).ToList());

            return testPaperRuleList;

        }

        private async Task<List<TestPaperItemDto>> GetTestPaperItemDtos(EntityQueryRequest entityQueryRequest)
        {
            var testPaperRuleTable = _testPaperItemRepository.Queryable().Where(x => x.TestPaperId == entityQueryRequest.Id);


            var queryable = testPaperRuleTable.Select(t => new TestPaperItemDto
            {
                Id = t.Id,
                QuestionId = t.QuestionId,
                DifficultyLevel = t.DifficultyLevel,
                Title = t.Title,
                TestPaperId = t.TestPaperId,
                QuestionType = t.QuestionType
            });

            return await queryable.ToListAsync();

        }

        private void ResolveTestPaperId(AddTestPaperDto actionRequest, string id)
        {
            actionRequest.TestPaperRuleDtos.ForEach(x =>
            {
                x.TestPaperId = id;
            });

            actionRequest.TestPaperItemDtos.ForEach(x => x.TestPaperId = id);
        }

        private async Task<List<TestPaperRuleTag>> GetTestPaperRuleTags(EntityQueryRequest entityQueryRequest)
        {
            var expression = entityQueryRequest.GetTestPaperRuleExpression();
            var testPageRuleTable = _testPaperRuleRepository.Queryable().Where(expression);
            var testPageRuleTagTable = _testPaperRuleTagRepository.Queryable();

            var query = testPageRuleTagTable.InnerJoin(testPageRuleTable, (t, r) => t.TestPaperRuleId == r.Id).Select((t, r) => t);

            return await query.ToListAsync();
        }

        private async Task<List<TestPaperQuestionCountViewResponse>> CalcuteRandomQuestionCount(TestPaperQuestionCountRequest testPaperQuestionCountRequest)
        {
            var testPaperQuestionCountViewResponses = new List<TestPaperQuestionCountViewResponse>();

            var expression = testPaperQuestionCountRequest.GetTestPaperRuleExpression();

            var testPaperRules = await _testPaperRuleRepository.Queryable().Where(expression).ToListAsync();

            if (testPaperRules != null)
            {
                testPaperRules.ForEach(x => testPaperQuestionCountViewResponses.Add(new TestPaperQuestionCountViewResponse
                {
                    QuestionType = x.QuestionType,
                    Count = x.Count
                }));
            }

            return testPaperQuestionCountViewResponses;
        }

        private async Task<List<TestPaperQuestionCountViewResponse>> CalcuteManualQuestionCount(TestPaperQuestionCountRequest testPaperQuestionCountRequest)
        {
            var testPaperQuestionCountViewResponses = new List<TestPaperQuestionCountViewResponse>();

            var expression = testPaperQuestionCountRequest.GetTestPaperItemsExpression();

            var testPaperItemTable =  _testPaperItemRepository.Queryable().Where(expression);

            testPaperQuestionCountViewResponses = await testPaperItemTable.GroupBy(t=>t.QuestionType).Select(t => new TestPaperQuestionCountViewResponse
                {
                    QuestionType = t.QuestionType,
                    Count = SqlFunc.AggregateCount(t.Id)
                }).ToListAsync();           


            return testPaperQuestionCountViewResponses;
        }



        private async Task AddTestPaperItemOptions(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            var questionOptionRepository = new ExamRepository<QuestionOptions>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);

            var quesitonIds = actionRequest.TestPaperItemDtos.Select(x => x.QuestionId);

            var questionOptions = await questionOptionRepository.Queryable().Where(x => quesitonIds.Contains(x.QuestionId)).ToListAsync();

            var testPaperItemOptions = new List<TestPaperItemOptions>();

            actionRequest.TestPaperItemDtos.Where(x => x.QuestionType.CheckSelectType()).ToList().ForEach(x =>
            {
                var options = questionOptions.Where(n => x.QuestionId == n.QuestionId).ToList();

                if (options != null)
                {
                    options.ForEach(item =>
                    {
                        var testPaperItemOption = _mapper.Map<QuestionOptions, TestPaperItemOptions>(item);

                        testPaperItemOption.ToInsert();

                        testPaperItemOptions.Add(testPaperItemOption);
                    });
                }

            });

            await _testPaperItemOptionsRepository.AddWithValidateAsync(testPaperItemOptions, cancellationToken);
        }

        private async Task AddTestPaperItemAnswer(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            var questionOptionRepository = new ExamRepository<QuestionAnswer>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);

            var quesitonIds = actionRequest.TestPaperItemDtos.Select(x => x.QuestionId);

            var questionAnswer = await questionOptionRepository.Queryable().Where(x => quesitonIds.Contains(x.QuestionId)).ToListAsync();

            var testPaperItemAnswer = new List<TestPaperItemAnswer>();

            actionRequest.TestPaperItemDtos.Where(x => !x.QuestionType.CheckSelectType()).ToList().ForEach(x =>
            {
                var options = questionAnswer.Where(n => x.QuestionId == n.QuestionId).ToList();

                options.ForEach(item =>
                {
                    var testPaperItemOption = _mapper.Map<QuestionAnswer, TestPaperItemAnswer>(item);

                    testPaperItemOption.ToInsert();

                    testPaperItemAnswer.Add(testPaperItemOption);
                });
            });

            await _testPaperItemAnswerRepository.AddWithValidateAsync(testPaperItemAnswer, cancellationToken);
        }




        private async Task AddTestPaperItemSourcewareAsync(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            var questionSourcewareRepository = new ExamRepository<QuestionSourceware>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);

            var quesitonIds = actionRequest.TestPaperItemDtos.Select(x => x.QuestionId);

            var questionSourceware = await questionSourcewareRepository.Queryable().Where(x => quesitonIds.Contains(x.QuestionId)).ToListAsync();

            var testPaperItemSourcewares = new List<TestPaperItemSourceware>();

            actionRequest.TestPaperItemDtos.Where(x => x.QuestionType.CheckSelectType()).ToList().ForEach(x =>
            {
                var options = questionSourceware.Where(n => x.QuestionId == n.QuestionId).ToList();

                if (options != null)
                {
                    options.ForEach(item =>
                    {
                        var testPaperItemSourceware = _mapper.Map<QuestionSourceware, TestPaperItemSourceware>(item);

                        testPaperItemSourceware.ToInsert();

                        testPaperItemSourcewares.Add(testPaperItemSourceware);
                    });
                }
            });

            await _testPaperItemSourcewareRepository.AddWithValidateAsync(testPaperItemSourcewares, cancellationToken);
        }

        private async Task AddTestPaperItemKnowladgeAsync(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            var questionKnowladgeRepository = new ExamRepository<QuestionKnowladge>(_repository.UOW, _dataPermissionFilterBuilder, _serviceProvider);

            var quesitonIds = actionRequest.TestPaperItemDtos.Select(x => x.QuestionId);

            var questionKnowladge = await questionKnowladgeRepository.Queryable().Where(x => quesitonIds.Contains(x.QuestionId)).ToListAsync();

            var testPaperItemKnowladges = new List<TestPaperItemKnowladge>();

            actionRequest.TestPaperItemDtos.Where(x => x.QuestionType.CheckSelectType()).ToList().ForEach(x =>
            {
                var options = questionKnowladge.Where(n => x.QuestionId == n.QuestionId).ToList();

                if (options != null)
                {
                    options.ForEach(item =>
                    {
                        var testPaperItemKnowladge = _mapper.Map<QuestionKnowladge, TestPaperItemKnowladge>(item);

                        testPaperItemKnowladge.ToInsert();

                        testPaperItemKnowladges.Add(testPaperItemKnowladge);
                    });
                }
            });

            await _testPaperItemKnowladgeRepository.AddWithValidateAsync(testPaperItemKnowladges, cancellationToken);
        }

        private async Task ModifyTestPaperItemSourcewareAsync(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperItemDtos == null) return;

            var questionIds = actionRequest.TestPaperItemDtos.Where(x => x.QuestionType.CheckSelectType()).Select(x => x.QuestionId).ToList();

            var entityQueryRequest = new EntityQueryRequest
            {
                Expression = ExpressionableUtility.CreateExpression<TestPaperItemSourceware>()
                             .AndIF(questionIds.IsNotNull(), x => questionIds.Contains(x.QuestionId)).ToExpression()
            };

            await DeleteTestPaperItemSourcewareAsync(entityQueryRequest, cancellationToken);

            await AddTestPaperItemSourcewareAsync(actionRequest, cancellationToken);
        }

        private async Task DeleteTestPaperItemSourcewareAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _testPaperItemSourcewareRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task ModifyTestPaperItemKnowladgeAsync(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperItemDtos == null) return;

            var questionIds = actionRequest.TestPaperItemDtos.Where(x => x.QuestionType.CheckSelectType()).Select(x => x.QuestionId).ToList();

            var entityQueryRequest = new EntityQueryRequest
            {
                Expression = ExpressionableUtility.CreateExpression<TestPaperItemKnowladge>()
                             .AndIF(questionIds.IsNotNull(), x => questionIds.Contains(x.QuestionId)).ToExpression()
            };

            await DeleteTestPaperItemKnowladgeAsync(entityQueryRequest, cancellationToken);

            await AddTestPaperItemKnowladgeAsync(actionRequest, cancellationToken);
        }

        private async Task DeleteTestPaperItemKnowladgeAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _testPaperItemKnowladgeRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }
        #endregion
    }
}
